@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
 # Copyright (C) 2012 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2017 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -27,12 +27,15 @@ BLACKLIST = {
     "renderer" :
         [
             "Software Rasterizer",
-            "Mesa DRI Intel(R) Ivybridge Desktop"
+            #"Mesa DRI Intel(R) Ivybridge Desktop"
         ],
     "vendor"    : [
         "VMware, Inc.",
-        "Humper",
+        #"Humper",
         #to disable nvidia, uncomment this:
         #"NVIDIA Corporation",
         ]
     }
+
+class OpenGLFatalError(ImportError):
+    pass
