@@ -1,31 +1,37 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2019 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, import_glib, is_gtk3
-gtk = import_gtk()
-gdk = import_gdk()
-glib = import_glib()
 import sys
 import datetime
+from collections import deque
 
 from xpra.version_util import XPRA_VERSION
 from xpra.os_util import bytestostr, strtobytes, get_linux_distribution, monotonic_time
 from xpra.util import prettify_plug_name, typedict, csv, engs
 from xpra.gtk_common.graph import make_graph_pixmap
-from collections import deque
 from xpra.simple_stats import values_to_scaled_values, values_to_diff_scaled_values, to_std_unit, std_unit_dec, std_unit
 from xpra.scripts.config import python_platform
-from xpra.log import Logger
-from xpra.gtk_common.gtk_util import add_close_accel, label, title_box, \
-                        TableBuilder, imagebutton, scaled_image, get_preferred_size, get_gtk_version_info, \
-                        RELIEF_NONE, RELIEF_NORMAL, EXPAND, FILL, WIN_POS_CENTER
+from xpra.client import mixin_features
+from xpra.gtk_common.gtk_util import (
+    add_close_accel, label, title_box, \
+    TableBuilder, imagebutton, scaled_image, get_preferred_size, get_gtk_version_info, \
+    RELIEF_NONE, RELIEF_NORMAL, EXPAND, FILL, WIN_POS_CENTER,
+    RESPONSE_CANCEL, RESPONSE_OK,
+    )
 from xpra.net.net_util import get_network_caps
+from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, import_glib, is_gtk3
+from xpra.log import Logger
+
 log = Logger("info")
 
+gtk = import_gtk()
+gdk = import_gdk()
+glib = import_glib()
+
 N_SAMPLES = 20      #how many sample points to show on the graphs
 SHOW_PIXEL_STATS = True
 SHOW_SOUND_STATS = True
@@ -75,8 +81,11 @@ def dictlook(d, k, fallback=None):
     v = d.get(k)
     if v is not None:
         return v
-    parts = strtobytes(k).split(b".")
+    parts = (b"client."+strtobytes(k)).split(b".")
     v = newdictlook(d, parts, fallback)
+    if v is None:
+        parts = strtobytes(k).split(b".")
+        v = newdictlook(d, parts, fallback)
     if v is None:
         return fallback
     return v
@@ -303,7 +312,7 @@ class SessionInfo(gtk.Window):
         tb, _ = self.table_tab("connect.png", "Connection", self.populate_connection)
         if self.connection:
             tb.new_row("Server Endpoint", slabel(self.connection.target))
-        if self.client.server_display:
+        if mixin_features.display and self.client.server_display:
             tb.new_row("Server Display", slabel(prettify_plug_name(self.client.server_display)))
         hostname = scaps.strget("hostname")
         if hostname:
@@ -358,17 +367,17 @@ class SessionInfo(gtk.Window):
         self.client_latency_labels = maths_labels()
         tb.add_row(slabel("Client Latency (ms)", "The time it takes for the client to respond to pings, as measured by the server"),
                    *self.client_latency_labels)
-        if self.client.windows_enabled:
+        if mixin_features.windows and self.client.windows_enabled:
             self.batch_labels = maths_labels()
-            tb.add_row(slabel("Batch Delay (ms)", "How long the server waits for new screen updates to accumulate before processing them"),
+            tb.add_row(slabel("Batch Delay (MPixels / ms)", "How long the server waits for new screen updates to accumulate before processing them"),
                        *self.batch_labels)
             self.damage_labels = maths_labels()
             tb.add_row(slabel("Damage Latency (ms)", "The time it takes to compress a frame and pass it to the OS network layer"),
                        *self.damage_labels)
             self.quality_labels = maths_labels()
-            tb.add_row(slabel("Encoding Quality (pct)"), *self.quality_labels)
+            tb.add_row(slabel("Encoding Quality (pct)", "Automatic picture quality, average for all the windows"), *self.quality_labels)
             self.speed_labels = maths_labels()
-            tb.add_row(slabel("Encoding Speed (pct)"), *self.speed_labels)
+            tb.add_row(slabel("Encoding Speed (pct)", "Automatic picture encoding speed (bandwidth vs CPU usage), average for all the windows"), *self.speed_labels)
 
             self.decoding_labels = maths_labels()
             tb.add_row(slabel("Decoding Latency (ms)", "How long it takes the client to decode a screen update"), *self.decoding_labels)
@@ -445,7 +454,7 @@ class SessionInfo(gtk.Window):
         self.populate_all()
         glib.timeout_add(1000, self.populate)
         glib.timeout_add(100, self.populate_tab)
-        if SHOW_SOUND_STATS:
+        if mixin_features.audio and SHOW_SOUND_STATS:
             glib.timeout_add(100, self.populate_sound_stats)
         add_close_accel(self, self.destroy)
 
@@ -571,16 +580,17 @@ class SessionInfo(gtk.Window):
         #record bytecount every second:
         self.net_in_bytecount.append(self.connection.input_bytecount)
         self.net_out_bytecount.append(self.connection.output_bytecount)
-        if SHOW_SOUND_STATS:
+        if mixin_features.audio and SHOW_SOUND_STATS:
             if self.client.sound_in_bytecount>0:
                 self.sound_in_bitcount.append(self.client.sound_in_bytecount * 8)
             if self.client.sound_out_bytecount>0:
                 self.sound_out_bitcount.append(self.client.sound_out_bytecount * 8)
 
-        #count pixels in the last second:
-        since = monotonic_time()-1
-        decoded = [0]+[pixels for _,t,pixels in self.client.pixel_counter if t>since]
-        self.pixel_in_data.append(sum(decoded))
+        if mixin_features.windows:
+            #count pixels in the last second:
+            since = monotonic_time()-1
+            decoded = [0]+[pixels for _,t,pixels in self.client.pixel_counter if t>since]
+            self.pixel_in_data.append(sum(decoded))
         #update latency values
         #there may be more than one record for each second
         #so we have to average them to prevent the graph from "jumping":
@@ -608,19 +618,22 @@ class SessionInfo(gtk.Window):
         self.client_latency = get_ping_latency_records(self.client.client_ping_latency)
         if self.client.server_last_info:
             #populate running averages for graphs:
-            def getavg(name):
-                return dictlook(self.client.server_last_info, "%s.avg" % name)
-            def addavg(l, name):
-                v = getavg(name)
+            def getavg(*names):
+                return (
+                    newdictlook(self.client.server_last_info, list(names)+["avg"]) or
+                    newdictlook(self.client.server_last_info, ["client"]+list(names)+["avg"])
+                    )
+            def addavg(l, *names):
+                v = getavg(*names)
                 if v:
                     l.append(v)
-            addavg(self.avg_batch_delay, "batch.delay")
-            addavg(self.avg_damage_out_latency, "damage.out_latency")
+            addavg(self.avg_batch_delay, "batch", "delay")
+            addavg(self.avg_damage_out_latency, "damage", "out_latency")
             if len(self.client.server_ping_latency)>0 and len(self.client.client_ping_latency)>0:
                 spl = [1000.0*x for _,x in tuple(self.client.server_ping_latency)]
                 cpl = [1000.0*x for _,x in tuple(self.client.client_ping_latency)]
                 self.avg_ping_latency.append(sum(spl+cpl)/len(spl+cpl))
-            if len(self.client.pixel_counter)>0:
+            if mixin_features.windows and len(self.client.pixel_counter)>0:
                 tsize = 0
                 ttime = 0
                 for start_time, end_time, size in self.client.pixel_counter:
@@ -682,6 +695,8 @@ class SessionInfo(gtk.Window):
         self.opengl_buffering.set_text(" ".join(info))
 
     def show_window_renderers(self):
+        if not mixin_features.windows:
+            return
         wr = []
         renderers = {}
         for wid, window in tuple(self.client._id_to_window.items()):
@@ -692,15 +707,21 @@ class SessionInfo(gtk.Window):
 
     def populate_features(self):
         size_info = ""
-        if self.client.server_actual_desktop_size:
-            w,h = self.client.server_actual_desktop_size
-            size_info = "%sx%s" % (w,h)
-            if self.client.server_randr and self.client.server_max_desktop_size:
-                size_info += " (max %s)" % ("x".join([str(x) for x in self.client.server_max_desktop_size]))
-        self.bool_icon(self.server_randr_icon, self.client.server_randr)
+        if mixin_features.windows:
+            if self.client.server_actual_desktop_size:
+                w,h = self.client.server_actual_desktop_size
+                size_info = "%sx%s" % (w,h)
+                if self.client.server_randr and self.client.server_max_desktop_size:
+                    size_info += " (max %s)" % ("x".join([str(x) for x in self.client.server_max_desktop_size]))
+                self.bool_icon(self.server_randr_icon, self.client.server_randr)
+        else:
+            size_info = "unknown"
+            unknown = self.get_pixbuf("unknown.png")
+            if unknown:
+                self.server_randr_icon.set_from_pixbuf(unknown)
         self.server_randr_label.set_text("%s" % size_info)
         root_w, root_h = self.client.get_root_size()
-        if self.client.xscale!=1 or self.client.yscale!=1:
+        if mixin_features.windows and (self.client.xscale!=1 or self.client.yscale!=1):
             sw, sh = self.client.cp(root_w, root_h)
             display_info = "%ix%i (scaled from %ix%i)" % (sw, sh, root_w, root_h)
         else:
@@ -734,10 +755,11 @@ class SessionInfo(gtk.Window):
             if not enabled:
                 return "n/a"
             return ", ".join(codecs or [])
-        self.server_speaker_codecs_label.set_text(codec_info(scaps.boolget("sound.send", False), scaps.strlistget("sound.encoders", [])))
-        self.client_speaker_codecs_label.set_text(codec_info(self.client.speaker_allowed, self.client.speaker_codecs))
-        self.server_microphone_codecs_label.set_text(codec_info(scaps.boolget("sound.receive", False), scaps.strlistget("sound.decoders", [])))
-        self.client_microphone_codecs_label.set_text(codec_info(self.client.microphone_allowed, self.client.microphone_codecs))
+        if mixin_features.audio:
+            self.server_speaker_codecs_label.set_text(codec_info(scaps.boolget("sound.send", False), scaps.strlistget("sound.encoders", [])))
+            self.client_speaker_codecs_label.set_text(codec_info(self.client.speaker_allowed, self.client.speaker_codecs))
+            self.server_microphone_codecs_label.set_text(codec_info(scaps.boolget("sound.receive", False), scaps.strlistget("sound.decoders", [])))
+            self.client_microphone_codecs_label.set_text(codec_info(self.client.microphone_allowed, self.client.microphone_codecs))
         def encliststr(v):
             v = list(v)
             try:
@@ -747,7 +769,10 @@ class SessionInfo(gtk.Window):
             return csv(sorted(v))
         se = scaps.strlistget("encodings.core", scaps.strlistget("encodings"))
         self.server_encodings_label.set_text(encliststr(se))
-        self.client_encodings_label.set_text(encliststr(self.client.get_core_encodings()))
+        if mixin_features.encoding:
+            self.client_encodings_label.set_text(encliststr(self.client.get_core_encodings()))
+        else:
+            self.client_encodings_label.set_text("n/a")
 
         def get_encoder_list(caps):
             from xpra.net import packet_encoding
@@ -764,7 +789,8 @@ class SessionInfo(gtk.Window):
 
     def populate_connection(self):
         def settimedeltastr(label, from_time):
-            delta = datetime.timedelta(seconds=(int(monotonic_time())-int(from_time)))
+            import time
+            delta = datetime.timedelta(seconds=(int(time.time())-int(from_time)))
             label.set_text(str(delta))
         if self.client.server_load:
             self.server_load_label.set_text("  ".join([str(x/1000.0) for x in self.client.server_load]))
@@ -784,32 +810,33 @@ class SessionInfo(gtk.Window):
         self.output_packets_label.set_text(std_unit_dec(p.output_packetcount))
         self.output_bytes_label.set_text(std_unit_dec(c.output_bytecount))
 
-        def get_sound_info(supported, prop):
-            if not supported:
-                return {"state" : "disabled"}
-            if prop is None:
-                return {"state" : "inactive"}
-            return prop.get_info()
-        def set_sound_info(label, details, supported, prop):
-            d = typedict(get_sound_info(supported, prop))
-            state = d.strget("state", "")
-            codec_descr = d.strget("codec") or d.strget("codec_description")
-            container_descr = d.strget("container_description", "")
-            if state=="active" and codec_descr:
-                if codec_descr.find(container_descr)>=0:
-                    descr = codec_descr
-                else:
-                    descr = csv(x for x in (codec_descr, container_descr) if x)
-                state = "%s: %s" % (state, descr)
-            label.set_text(state)
-            if details:
-                s = ""
-                bitrate = d.intget("bitrate", 0)
-                if bitrate>0:
-                    s = "%sbit/s" % std_unit(bitrate)
-                details.set_text(s)
-        set_sound_info(self.speaker_label, self.speaker_details, self.client.speaker_enabled, self.client.sound_sink)
-        set_sound_info(self.microphone_label, None, self.client.microphone_enabled, self.client.sound_source)
+        if mixin_features.audio:
+            def get_sound_info(supported, prop):
+                if not supported:
+                    return {"state" : "disabled"}
+                if prop is None:
+                    return {"state" : "inactive"}
+                return prop.get_info()
+            def set_sound_info(label, details, supported, prop):
+                d = typedict(get_sound_info(supported, prop))
+                state = d.strget("state", "")
+                codec_descr = d.strget("codec") or d.strget("codec_description")
+                container_descr = d.strget("container_description", "")
+                if state=="active" and codec_descr:
+                    if codec_descr.find(container_descr)>=0:
+                        descr = codec_descr
+                    else:
+                        descr = csv(x for x in (codec_descr, container_descr) if x)
+                    state = "%s: %s" % (state, descr)
+                label.set_text(state)
+                if details:
+                    s = ""
+                    bitrate = d.intget("bitrate", 0)
+                    if bitrate>0:
+                        s = "%sbit/s" % std_unit(bitrate)
+                    details.set_text(s)
+            set_sound_info(self.speaker_label, self.speaker_details, self.client.speaker_enabled, self.client.sound_sink)
+            set_sound_info(self.microphone_label, None, self.client.microphone_enabled, self.client.sound_source)
 
         self.connection_type_label.set_text(c.socktype)
         protocol_info = p.get_info()
@@ -852,10 +879,28 @@ class SessionInfo(gtk.Window):
         return getv("cur"), getv("min"), getv("avg"), getv("90p"), getv("max")
 
     def all_values_from_info(self, *window_props):
+        #newer (2.4 and later) servers can just give us the value directly:
+        for window_prop in window_props:
+            prop_path = "client.%s" % window_prop
+            v = dictlook(self.client.server_last_info, prop_path)
+            if v is not None:
+                try:
+                    v = typedict(v)
+                except TypeError:
+                    #backwards compatibility:
+                    #older servers don't expose the correct value or type here
+                    #so don't use this value
+                    log("expected dictionary for %s", prop_path)
+                    log(" got %s: %s", type(v), v)
+                else:
+                    iget = v.intget
+                    return iget("cur"), iget("min"), iget("avg"), iget("90p"), iget("max")
+
+        #legacy servers: sum up the values for all the windows found
         def avg(values):
             if not values:
                 return ""
-            return sum(values) / len(values)
+            return sum(values) // len(values)
         def getv(suffix, op):
             if self.client.server_last_info is None:
                 return ""
@@ -890,12 +935,10 @@ class SessionInfo(gtk.Window):
         self.client.send_info_request()
         def setall(labels, values):
             assert len(labels)==len(values), "%s labels and %s values (%s vs %s)" % (len(labels), len(values), labels, values)
-            for i in range(len(labels)):
-                l = labels[i]
-                v = values[i]
-                l.set_text(str(v))
+            for i, l in enumerate(labels):
+                l.set_text(str(values[i]))
         def setlabels(labels, values, rounding=int):
-            if len(values)==0:
+            if not values:
                 return
             avg = sum(values)/len(values)
             svalues = sorted(values)
@@ -911,13 +954,13 @@ class SessionInfo(gtk.Window):
             rounded_values = [rounding(v) for v in disp]
             setall(labels, rounded_values)
 
-        if len(self.client.server_ping_latency)>0:
+        if self.client.server_ping_latency:
             spl = [1000.0*x for _,x in tuple(self.client.server_ping_latency)]
             setlabels(self.server_latency_labels, spl)
-        if len(self.client.client_ping_latency)>0:
+        if self.client.client_ping_latency:
             cpl = [1000.0*x for _,x in tuple(self.client.client_ping_latency)]
             setlabels(self.client_latency_labels, cpl)
-        if self.client.windows_enabled:
+        if mixin_features.windows and self.client.windows_enabled:
             setall(self.batch_labels, self.values_from_info("batch_delay", "batch.delay"))
             setall(self.damage_labels, self.values_from_info("damage_out_latency", "damage.out_latency"))
             setall(self.quality_labels, self.all_values_from_info("quality", "encoding.quality"))
@@ -927,7 +970,7 @@ class SessionInfo(gtk.Window):
             rps = []
             pps = []
             decoding_latency = []
-            if len(self.client.pixel_counter)>0:
+            if self.client.pixel_counter:
                 min_time = None
                 max_time = None
                 regions_per_second = {}
@@ -978,7 +1021,7 @@ class SessionInfo(gtk.Window):
                 for wid, props in window_encoder_stats.items():
                     l = slabel("%s (%s)" % (wid, bytestostr(props.get(""))))
                     l.show()
-                    info = ["%s=%s" % (k,v) for k,v in props.items() if k!=""]
+                    info = ("%s=%s" % (k,v) for k,v in props.items() if k!="")
                     l.set_tooltip_text(" ".join(info))
                     self.encoder_info_box.add(l)
         return True
@@ -1026,7 +1069,9 @@ class SessionInfo(gtk.Window):
 
 
     def populate_graphs(self, *_args):
-        self.client.send_info_request("network", "damage", "state", "batch")
+        #older servers have 'batch' at top level,
+        #newer servers store it under client
+        self.client.send_info_request("network", "damage", "state", "batch", "client")
         box = self.tab_box
         _, h = get_preferred_size(box)
         _, bh = get_preferred_size(self.tab_button_box)
@@ -1060,15 +1105,15 @@ class SessionInfo(gtk.Window):
             else:
                 labels += ["recv %sB/s" % unit(net_in_scale)]
                 datasets += [net_in_data]
-        if SHOW_PIXEL_STATS and self.client.windows_enabled:
+        if mixin_features.windows and SHOW_PIXEL_STATS and self.client.windows_enabled:
             pixel_scale, in_pixels = values_to_scaled_values(tuple(self.pixel_in_data)[3:N_SAMPLES+4], min_scaled_value=100)
             datasets.append(in_pixels)
             labels.append("%s pixels/s" % unit(pixel_scale))
-        if SHOW_SOUND_STATS and self.sound_in_bitcount:
+        if mixin_features.audio and SHOW_SOUND_STATS and self.sound_in_bitcount:
             sound_in_scale, sound_in_data = values_to_diff_scaled_values(tuple(self.sound_in_bitcount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
             datasets.append(sound_in_data)
             labels.append("Speaker %sb/s" % unit(sound_in_scale))
-        if SHOW_SOUND_STATS and self.sound_out_bitcount:
+        if mixin_features.audio and SHOW_SOUND_STATS and self.sound_out_bitcount:
             sound_out_scale, sound_out_data = values_to_diff_scaled_values(tuple(self.sound_out_bitcount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
             datasets.append(sound_out_data)
             labels.append("Mic %sb/s" % unit(sound_out_scale))
@@ -1086,7 +1131,7 @@ class SessionInfo(gtk.Window):
             #(and skip if we don't have any)
             values, labels = [], []
             for l, name in items:
-                if len(l)==0:
+                if not l:
                     continue
                 l = list(l)
                 if len(l)<size:
@@ -1113,7 +1158,7 @@ class SessionInfo(gtk.Window):
         self.latency_graph.set_size_request(W, H)
         self.latency_graph.set_from_pixmap(pixmap, None)
 
-        if SHOW_SOUND_STATS and self.client.sound_sink:
+        if mixin_features.audio and SHOW_SOUND_STATS and self.client.sound_sink:
             #sound queue graph:
             queue_values, queue_labels = norm_lists((
                                  (self.sound_out_queue_max, "Max"),
@@ -1132,7 +1177,7 @@ class SessionInfo(gtk.Window):
         log("save_graph(%s)", args)
         chooser = gtk.FileChooserDialog("Save graphs as a PNG image",
                                     parent=self, action=gtk.FILE_CHOOSER_ACTION_SAVE,
-                                    buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK))
+                                    buttons=(gtk.STOCK_CANCEL, RESPONSE_CANCEL, gtk.STOCK_SAVE, RESPONSE_OK))
         chooser.set_select_multiple(False)
         chooser.set_default_response(gtk.RESPONSE_OK)
         file_filter = gtk.FileFilter()
@@ -1146,7 +1191,8 @@ class SessionInfo(gtk.Window):
         if response == gtk.RESPONSE_OK:
             if len(filenames)==1:
                 filename = filenames[0]
-                pixmaps = [image.get_pixmap()[0] for image in [self.bandwidth_graph, self.latency_graph, self.sound_queue_graph]]
+                pixmaps = tuple(image.get_pixmap()[0] for image in
+                                (self.bandwidth_graph, self.latency_graph, self.sound_queue_graph))
                 log("saving pixmaps %s and %s to %s", pixmaps, filename)
                 w, h = 0, 0
                 for pixmap in pixmaps:
