@@ -1,25 +1,27 @@
 # This file is part of Xpra.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import hashlib
-from xpra.log import Logger
-log = Logger("paint")
-deltalog = Logger("delta")
-
 from threading import Lock
+from PIL import Image
+
 from xpra.net.mmap_pipe import mmap_read
 from xpra.net import compression
 from xpra.util import typedict, csv, envint, envbool, repr_ellipsized
-from xpra.codecs.loader import get_codec
+from xpra.codecs.loader import get_codec, is_loaded
 from xpra.codecs.video_helper import getVideoHelper
 from xpra.os_util import BytesIOClass, bytestostr, _buffer
 try:
     from xpra.codecs.xor.cyxor import xor_str   #@UnresolvedImport
 except ImportError:
     from xpra.util import xor as xor_str
+from xpra.log import Logger
+
+log = Logger("paint")
+deltalog = Logger("delta")
 
 DELTA_BUCKETS = envint("XPRA_DELTA_BUCKETS", 5)
 INTEGRITY_HASH = envbool("XPRA_INTEGRITY_HASH", False)
@@ -50,7 +52,7 @@ def load_video_decoders():
             specs = vh.get_decoder_specs(encoding)
             for colorspace, decoders in specs.items():
                 log("%-5s decoders for %7s: %s", encoding, colorspace, csv([d.get_type() for _,d in decoders]))
-                assert len(decoders)>0
+                assert decoders
                 #use the first one:
                 _, decoder_module = decoders[0]
                 VIDEO_DECODERS[encoding] = decoder_module
@@ -73,6 +75,8 @@ Generic superclass for all Backing code,
 see CairoBackingBase and GTK2WindowBacking subclasses for actual implementations
 """
 class WindowBackingBase(object):
+    RGB_MODES = ()
+
     def __init__(self, wid, window_alpha):
         load_csc_options()
         load_video_decoders()
@@ -92,14 +96,17 @@ class WindowBackingBase(object):
         self.pointer_overlay = None
         self.cursor_data = None
         self.default_cursor_data = None
-        PIL = get_codec("dec_pillow")
-        if PIL:
-            self._PIL_encodings = PIL.get_encodings()
+        self.jpeg_decoder = None
+        self.webp_decoder = None
+        if is_loaded():
+            PIL = get_codec("dec_pillow")
+            if PIL:
+                self._PIL_encodings = PIL.get_encodings()
+            self.jpeg_decoder = get_codec("dec_jpeg")
+            self.webp_decoder = get_codec("dec_webp")
         self.draw_needs_refresh = True
         self.mmap = None
         self.mmap_enabled = False
-        self.jpeg_decoder = get_codec("dec_jpeg")
-        self.webp_decoder = get_codec("dec_webp")
 
     def idle_add(self, *_args, **_kwargs):
         raise NotImplementedError()
@@ -189,7 +196,8 @@ class WindowBackingBase(object):
                 assert len(comp)==1, "more than one compressor specified: %s" % str(comp)
                 img_data = compression.decompress_by_name(raw_data, algo=comp[0])
         if len(img_data)!=rowstride * height:
-            deltalog.error("invalid img data length: expected %s but got %s (%s: %s)", rowstride * height, len(img_data), type(img_data), repr_ellipsized(img_data))
+            deltalog.error("Error: invalid img data length: expected %s but got %s (%s: %s)",
+                           rowstride * height, len(img_data), type(img_data), repr_ellipsized(img_data))
             raise Exception("expected %s bytes for %sx%s with rowstride=%s but received %s (%s compressed)" %
                                 (rowstride * height, width, height, rowstride, len(img_data), len(raw_data)))
         delta = options.intget(b"delta", -1)
@@ -197,12 +205,13 @@ class WindowBackingBase(object):
         rgb_format = options.strget(b"rgb_format")
         rgb_data = img_data
         if delta>=0:
-            assert bucket>=0 and bucket<DELTA_BUCKETS, "invalid delta bucket number: %s" % bucket
+            assert 0<=bucket<DELTA_BUCKETS, "invalid delta bucket number: %s" % bucket
             if self._delta_pixel_data[bucket] is None:
                 raise Exception("delta region bucket %s references pixmap data we do not have!" % bucket)
             lwidth, lheight, lrgb_format, seq, ldata = self._delta_pixel_data[bucket]
             assert width==lwidth and height==lheight and delta==seq, \
-                "delta bucket %s data does not match: expected %s but got %s" % (bucket, (width, height, delta), (lwidth, lheight, seq))
+                "delta bucket %s data does not match: expected %s but got %s" % (
+                    bucket, (width, height, delta), (lwidth, lheight, seq))
             assert lrgb_format==rgb_format, "delta region uses %s format, was expecting %s" % (rgb_format, lrgb_format)
             deltalog("delta: xoring with bucket %i", bucket)
             rgb_data = xor_str(img_data, ldata)
@@ -226,11 +235,8 @@ class WindowBackingBase(object):
 
     def paint_image(self, coding, img_data, x, y, width, height, options, callbacks):
         """ can be called from any thread """
-        #log("paint_image(%s, %s bytes, %s, %s, %s, %s, %s, %s)", coding, len(img_data), x, y, width, height, options, callbacks)
-        PIL = get_codec("PIL")
-        assert PIL.Image, "PIL.Image not found"
         buf = BytesIOClass(img_data)
-        img = PIL.Image.open(buf)
+        img = Image.open(buf)
         assert img.mode in ("L", "P", "RGB", "RGBA", "RGBX"), "invalid image mode: %s" % img.mode
         transparency = options.intget("transparency", -1)
         if img.mode=="P":
@@ -246,13 +252,13 @@ class WindowBackingBase(object):
                     if a!=transparency:
                         return 255
                     return 0
-                mask = PIL.Image.eval(img, mask_value)
+                mask = Image.eval(img, mask_value)
                 mask = mask.convert("L")
                 def nomask_value(a):
                     if a!=transparency:
                         return a
                     return 0
-                img = PIL.Image.eval(img, nomask_value)
+                img = Image.eval(img, nomask_value)
                 img = img.convert("RGBA")
                 img.putalpha(mask)
             else:
@@ -300,7 +306,11 @@ class WindowBackingBase(object):
             return self.paint_image("webp", img_data, x, y, width, height, options, callbacks)
         rgb_format = options.strget("rgb_format")
         has_alpha = options.boolget("has_alpha", False)
-        buffer_wrapper, width, height, stride, has_alpha, rgb_format = self.webp_decoder.decompress(img_data, has_alpha, rgb_format, self.RGB_MODES)
+        (
+            buffer_wrapper,
+            width, height, stride, has_alpha,
+            rgb_format,
+            ) = self.webp_decoder.decompress(img_data, has_alpha, rgb_format, self.RGB_MODES)
         def free_buffer(*_args):
             buffer_wrapper.free()
         callbacks.append(free_buffer)
@@ -310,8 +320,8 @@ class WindowBackingBase(object):
         if rgb_format not in self.RGB_MODES:
             from xpra.codecs.rgb_transform import rgb_reformat
             from xpra.codecs.image_wrapper import ImageWrapper
-            img = ImageWrapper(x, y, width, height, data, rgb_format, len(rgb_format)*8, stride, len(rgb_format), ImageWrapper.PACKED, True, None)
-            #log("rgb_reformat(%s, %s, %s) %i bytes", img, self.RGB_MODES, has_alpha and self._alpha_enabled, len(img_data))
+            img = ImageWrapper(x, y, width, height, data, rgb_format,
+                               len(rgb_format)*8, stride, len(rgb_format), ImageWrapper.PACKED, True, None)
             rgb_reformat(img, self.RGB_MODES, has_alpha and self._alpha_enabled)
             rgb_format = img.get_pixel_format()
             data = img.get_pixels()
@@ -346,8 +356,6 @@ class WindowBackingBase(object):
             options[b"rgb_format"] = rgb_format
             success = paint_fn(img_data, x, y, width, height, rowstride, options)
             fire_paint_callbacks(callbacks, success)
-        except KeyboardInterrupt:
-            raise
         except Exception as e:
             if not self._backing:
                 fire_paint_callbacks(callbacks, -1, "paint error on closed backing ignored")
@@ -374,10 +382,11 @@ class WindowBackingBase(object):
                        dst_width, dst_height, dst_format_options, speed):
         global CSC_OPTIONS
         in_options = CSC_OPTIONS.get(src_format, {})
-        assert len(in_options)>0, "no csc options for '%s' input in %s" % (src_format, CSC_OPTIONS)
+        assert in_options, "no csc options for '%s' input in %s" % (src_format, CSC_OPTIONS)
         for dst_format in dst_format_options:
             specs = in_options.get(dst_format)
-            log("make_csc%s specs=%s", (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, speed), specs)
+            log("make_csc%s specs=%s",
+                (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, speed), specs)
             if not specs:
                 continue
             for spec in specs:
@@ -390,7 +399,9 @@ class WindowBackingBase(object):
                                dst_width, dst_height, dst_format, speed)
                     return csc
                 except Exception as e:
-                    log("make_csc%s", (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, speed), exc_info=True)
+                    log("make_csc%s",
+                        (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, speed),
+                        exc_info=True)
                     log.error("Error: failed to create csc instance %s", spec.codec_class)
                     log.error(" for %s to %s: %s", src_format, dst_format, e)
         log.error("Error: no matching CSC module found")
@@ -408,31 +419,32 @@ class WindowBackingBase(object):
                 v = self.validate_csc_size(spec, src_width, src_height, dst_width, dst_height)
                 if v:
                     log.error("       "+v[0], *v[1:])
-        raise Exception("no csc module found for %s(%sx%s) to %s(%sx%s)" % (src_format, src_width, src_height, " or ".join(dst_format_options), dst_width, dst_height, CSC_OPTIONS))
+        raise Exception("no csc module found for %s(%sx%s) to %s(%sx%s) in %s" %
+                        (src_format, src_width, src_height, " or ".join(dst_format_options),
+                         dst_width, dst_height, CSC_OPTIONS))
 
     def validate_csc_size(self, spec, src_width, src_height, dst_width, dst_height):
         if not spec.can_scale and (src_width!=dst_width or src_height!=dst_height):
             return "scaling not suported"
-        elif src_width<spec.min_w:
+        if src_width<spec.min_w:
             return "source width %i is out of range: minimum is %i", src_width, spec.min_w
-        elif src_height<spec.min_h:
+        if src_height<spec.min_h:
             return "source height %i is out of range: minimum is %i", src_height, spec.min_h
-        elif dst_width<spec.min_w:
+        if dst_width<spec.min_w:
             return "target width %i is out of range: minimum is %i", dst_width, spec.min_w
-        elif dst_height<spec.min_h:
+        if dst_height<spec.min_h:
             return "target height %i is out of range: minimum is %i", dst_height, spec.min_h
-        elif src_width>spec.max_w:
+        if src_width>spec.max_w:
             return "source width %i is out of range: maximum is %i", src_width, spec.max_w
-        elif src_height>spec.max_h:
+        if src_height>spec.max_h:
             return "source height %i is out of range: maximum is %i", src_height, spec.max_h
-        elif dst_width>spec.max_w:
+        if dst_width>spec.max_w:
             return "target width %i is out of range: maximum is %i", dst_width, spec.max_w
-        elif dst_height>spec.max_h:
+        if dst_height>spec.max_h:
             return "target height %i is out of range: maximum is %i", dst_height, spec.max_h
         return None
 
     def paint_with_video_decoder(self, decoder_module, coding, img_data, x, y, width, height, options, callbacks):
-        #log("paint_with_video_decoder%s", (decoder_module, coding, "%s bytes" % len(img_data), x, y, width, height, options, callbacks))
         assert decoder_module, "decoder module not found for %s" % coding
         dl = self._decoder_lock
         if dl is None:
@@ -464,14 +476,17 @@ class WindowBackingBase(object):
                     log("paint_with_video_decoder: encoding changed from %s to %s", vd.get_encoding(), coding)
                     self.do_clean_video_decoder()
                 elif vd.get_width()!=enc_width or vd.get_height()!=enc_height:
-                    log("paint_with_video_decoder: video dimensions have changed from %s to %s", (vd.get_width(), vd.get_height()), (enc_width, enc_height))
+                    log("paint_with_video_decoder: video dimensions have changed from %s to %s",
+                        (vd.get_width(), vd.get_height()), (enc_width, enc_height))
                     self.do_clean_video_decoder()
                 elif vd.get_colorspace()!=input_colorspace:
                     #this should only happen on encoder restart, which means this should be the first frame:
-                    log.warn("Warning: colorspace unexpectedly changed from %s to %s", vd.get_colorspace(), input_colorspace)
+                    log.warn("Warning: colorspace unexpectedly changed from %s to %s",
+                             vd.get_colorspace(), input_colorspace)
                     self.do_clean_video_decoder()
             if self._video_decoder is None:
-                log("paint_with_video_decoder: new %s(%s,%s,%s)", decoder_module.Decoder, width, height, input_colorspace)
+                log("paint_with_video_decoder: new %s(%s,%s,%s)",
+                    decoder_module.Decoder, width, height, input_colorspace)
                 vd = decoder_module.Decoder()
                 vd.init_context(coding, enc_width, enc_height, input_colorspace)
                 self._video_decoder = vd
@@ -484,7 +499,9 @@ class WindowBackingBase(object):
                     #and this frame references those, so assume all is well:
                     fire_paint_callbacks(callbacks)
                 else:
-                    fire_paint_callbacks(callbacks, False, "video decoder %s failed to decode %i bytes of %s data" % (vd.get_type(), len(img_data), coding))
+                    fire_paint_callbacks(callbacks, False,
+                                         "video decoder %s failed to decode %i bytes of %s data" % (
+                                             vd.get_type(), len(img_data), coding))
                     log.error("Error: decode failed on %s bytes of %s data", len(img_data), coding)
                     log.error(" %sx%s pixels using %s", width, height, vd.get_type())
                     log.error(" frame options:")
@@ -552,7 +569,7 @@ class WindowBackingBase(object):
         #Note: BGR(A) is only handled by gl_window_backing
         self.do_paint_rgb(rgb_format, data, x, y, width, height, rowstride, options, callbacks)
 
-    def paint_scroll(self, *args):
+    def paint_scroll(self, _img_data, _options, callbacks):
         raise NotImplementedError("no paint scroll on %s" % type(self))
 
 
@@ -560,7 +577,8 @@ class WindowBackingBase(object):
         """ dispatches the paint to one of the paint_XXXX methods """
         try:
             assert self._backing is not None
-            log("draw_region(%s, %s, %s, %s, %s, %s bytes, %s, %s, %s)", x, y, width, height, coding, len(img_data), rowstride, options, callbacks)
+            log("draw_region(%s, %s, %s, %s, %s, %s bytes, %s, %s, %s)",
+                x, y, width, height, coding, len(img_data), rowstride, options, callbacks)
             coding = bytestostr(coding)
             options["encoding"] = coding            #used for choosing the color of the paint box
             if INTEGRITY_HASH:
@@ -575,22 +593,21 @@ class WindowBackingBase(object):
                 deltalog("passed compressed data integrity checks: len=%s, md5=%s (type=%s)", l, md5, type(img_data))
             if coding == "mmap":
                 self.idle_add(self.paint_mmap, img_data, x, y, width, height, rowstride, options, callbacks)
-            elif coding == "rgb24" or coding == "rgb32":
+            elif coding in ("rgb24", "rgb32"):
                 #avoid confusion over how many bytes-per-pixel we may have:
                 rgb_format = options.strget(b"rgb_format")
-                if rgb_format:
-                    Bpp = len(rgb_format)
-                elif coding=="rgb24":
-                    #legacy:
-                    rgb_format = "RGB"
-                else:
-                    #legacy:
-                    rgb_format = "RGBX"
+                if not rgb_format:
+                    rgb_format = {
+                        "rgb24" : "RGB",
+                        "rgb32" : "RGBX",
+                        }.get(coding)
                 if rowstride==0:
-                    rowstride = width * Bpp
+                    rowstride = width * len(rgb_format)
                 self.paint_rgb(rgb_format, img_data, x, y, width, height, rowstride, options, callbacks)
             elif coding in VIDEO_DECODERS:
-                self.paint_with_video_decoder(VIDEO_DECODERS.get(coding), coding, img_data, x, y, width, height, options, callbacks)
+                self.paint_with_video_decoder(VIDEO_DECODERS.get(coding),
+                                              coding,
+                                              img_data, x, y, width, height, options, callbacks)
             elif self.jpeg_decoder and coding=="jpeg":
                 self.paint_jpeg(img_data, x, y, width, height, options, callbacks)
             elif coding == "webp":
