@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2013-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -11,60 +11,66 @@ import sys
 import signal
 import uuid
 import time
+import struct
 import binascii
 
 #hide some ugly python3 compat:
 try:
     import _thread as thread            #@UnresolvedImport @UnusedImport (python3)
-except:
+except ImportError:
     import thread                       #@Reimport @UnusedImport
+assert thread
 
 try:
     from queue import Queue             #@UnresolvedImport @UnusedImport (python3)
 except ImportError:
     from Queue import Queue             #@Reimport @UnusedImport
+assert Queue
 
 try:
     import builtins                     #@UnresolvedImport @UnusedImport (python3)
-except:
+except ImportError:
     import __builtin__ as builtins      #@Reimport @UnusedImport
+assert builtins
 _buffer = builtins.__dict__.get("buffer")
 
 
 SIGNAMES = {}
-for x in [x for x in dir(signal) if x.startswith("SIG") and not x.startswith("SIG_")]:
-    SIGNAMES[getattr(signal, x)] = x
+for signame in (sig for sig in dir(signal) if sig.startswith("SIG") and not sig.startswith("SIG_")):
+    SIGNAMES[getattr(signal, signame)] = signame
 
 
 #use cStringIO, fallback to StringIO,
 #and python3 is making life more difficult yet again:
 try:
     from io import BytesIO as BytesIOClass              #@UnusedImport
-except:
+except ImportError:
     try:
         from cStringIO import StringIO as BytesIOClass  #@Reimport @UnusedImport
-    except:
+    except ImportError:
         from StringIO import StringIO as BytesIOClass   #@Reimport @UnusedImport
+assert BytesIOClass
 try:
     from StringIO import StringIO as StringIOClass      #@UnusedImport
-except:
+except ImportError:
     try:
         from cStringIO import StringIO as StringIOClass #@Reimport @UnusedImport
-    except:
+    except ImportError:
         from io import StringIO as StringIOClass        #@Reimport @UnusedImport
-
+assert StringIOClass
 
 WIN32 = sys.platform.startswith("win")
 OSX = sys.platform.startswith("darwin")
 LINUX = sys.platform.startswith("linux")
+NETBSD = sys.platform.startswith("netbsd")
+OPENBSD = sys.platform.startswith("openbsd")
 FREEBSD  = sys.platform.startswith("freebsd")
 
 POSIX = os.name=="posix"
 PYTHON2 = sys.version_info[0]==2
 PYTHON3 = sys.version_info[0]==3
 
-import struct
-BITS = struct.calcsize("P")*8
+BITS = struct.calcsize(b"P")*8
 
 
 if PYTHON2:
@@ -76,11 +82,11 @@ if PYTHON2:
         return binascii.hexlify(str(v))
 else:
     def strtobytes(x):
-        if type(x)==bytes:
+        if isinstance(x, bytes):
             return x
         return str(x).encode("latin1")
     def bytestostr(x):
-        if type(x)==bytes:
+        if isinstance(x, bytes):
             return x.decode("latin1")
         return str(x)
     def hexstr(v):
@@ -96,7 +102,7 @@ def get_util_logger():
     return util_logger
 
 def memoryview_to_bytes(v):
-    if type(v)==bytes:
+    if isinstance(v, bytes):
         return v
     if isinstance(v, memoryview):
         return v.tobytes()
@@ -161,7 +167,7 @@ def get_group_id(group):
         import grp      #@UnresolvedImport
         gr = grp.getgrnam(group)
         return gr.gr_gid
-    except:
+    except (ImportError, KeyError):
         return -1
 
 
@@ -184,7 +190,7 @@ def platform_release(release):
     return release
 
 
-def platform_name(sys_platform, release):
+def platform_name(sys_platform, release=None):
     if not sys_platform:
         return "unknown"
     PLATFORMS = {"win32"    : "Microsoft Windows",
@@ -196,7 +202,7 @@ def platform_name(sys_platform, release):
                  }
     def rel(v):
         values = [v]
-        if type(release) in (tuple, list):
+        if isinstance(release, (tuple, list)):
             values += list(release)
         else:
             values.append(release)
@@ -230,16 +236,18 @@ def get_machine_id():
             v = load_binary_file(filename)
             if v is not None:
                 break
-    return  str(v).strip("\n\r")
+    elif WIN32:
+        v = uuid.getnode()
+    return bytestostr(v).strip("\n\r")
 
 def get_user_uuid():
     """
         Try to generate a uuid string which is unique to this user.
         (relies on get_machine_id to uniquely identify a machine)
     """
-    uuid = os.environ.get("XPRA_USER_UUID")
-    if uuid:
-        return uuid
+    user_uuid = os.environ.get("XPRA_USER_UUID")
+    if user_uuid:
+        return user_uuid
     import hashlib
     u = hashlib.sha1()
     def uupdate(ustr):
@@ -250,17 +258,29 @@ def get_user_uuid():
         uupdate(str(os.getuid()))
         uupdate(u"/")
         uupdate(str(os.getgid()))
-    uupdate(os.environ.get("HOME", ""))
+    uupdate(os.path.expanduser("~/"))
     return u.hexdigest()
 
 
-monotonic_time = time.time
 try:
     from xpra.monotonic_time import _monotonic_time     #@UnresolvedImport
     assert _monotonic_time()>0
     monotonic_time = _monotonic_time
-except Exception as e:
-    pass
+except (ImportError, AssertionError):
+    monotonic_time = time.time
+
+
+def is_X11():
+    if OSX or WIN32:
+        return False
+    if PYTHON2:
+        return True
+    from xpra.x11.gtk3.gdk_bindings import is_X11_Display   #@UnresolvedImport
+    return is_X11_Display()
+
+def is_Wayland():
+    return os.environ.get("WAYLAND_DISPLAY") or os.environ.get("XDG_SESSION_TYPE")=="wayland"
+
 
 def is_distribution_variant(variant=b"Debian", os_file="/etc/os-release"):
     if not POSIX:
@@ -308,7 +328,7 @@ def get_linux_distribution():
         cmd = ["lsb_release", "-a"]
         try:
             p = subprocess.Popen(cmd, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-            out, _ = p.communicate()
+            out = p.communicate()[0]
             assert p.returncode==0 and out
         except:
             try:
@@ -333,26 +353,65 @@ def getUbuntuVersion():
     if distro and len(distro)==3 and distro[0]=="Ubuntu":
         ur = distro[1]  #ie: "12.04"
         try:
-            rnum = [int(x) for x in ur.split(".")]  #ie: [12, 4]
-            return rnum
-        except:
+            return tuple(int(x) for x in ur.split("."))  #ie: (12, 4)
+        except ValueError:
             pass
-    return []
+    return ()
 
 def is_unity():
-    return os.environ.get("XDG_CURRENT_DESKTOP", "").lower().startswith("unity")
+    return os.environ.get("XDG_CURRENT_DESKTOP", "").lower().find("unity")>=0
+
+def is_gnome():
+    return os.environ.get("XDG_CURRENT_DESKTOP", "").lower().find("gnome")>=0
+
+def is_kde():
+    return os.environ.get("XDG_CURRENT_DESKTOP", "").lower().find("kde")>=0
+
+
+def is_WSL():
+    if not POSIX:
+        return False
+    r = None
+    for f in ("/proc/sys/kernel/osrelease", "/proc/version"):
+        r = load_binary_file(f)
+        if r:
+            break
+    return r is not None and r.find(b"Microsoft")>=0
+
 
 def get_generic_os_name():
     for k,v in {
-        "linux"     : "linux",
-        "darwin"    : "osx",
-        "win"       : "win32",
-        "freebsd"   : "freebsd",
-        }.items():
+            "linux"     : "linux",
+            "darwin"    : "osx",
+            "win"       : "win32",
+            "freebsd"   : "freebsd",
+            }.items():
         if sys.platform.startswith(k):
             return v
     return sys.platform
 
+def get_cpu_count():
+    #sensible default:
+    cpus = 2
+    try:
+        try:
+            #python3:
+            cpus = os.cpu_count()
+        except AttributeError:
+            #python2:
+            import multiprocessing
+            cpus = multiprocessing.cpu_count()
+    except:
+        pass
+    return cpus
+
+
+def filedata_nocrlf(filename):
+    v = load_binary_file(filename)
+    if v is None:
+        get_util_logger().error("failed to load '%s'", filename)
+        return None
+    return v.strip("\n\r")
 
 def load_binary_file(filename):
     if not os.path.exists(filename):
@@ -374,27 +433,28 @@ def livefds():
     live = set()
     try:
         MAXFD = os.sysconf("SC_OPEN_MAX")
-    except:
+    except ValueError:
         MAXFD = 256
     for fd in range(0, MAXFD):
         try:
             s = os.fstat(fd)
+        except Exception:
+            continue
+        else:
             if s:
                 live.add(fd)
-        except:
-            continue
     return live
 
-def close_fds(excluding=[0, 1, 2]):
+def close_fds(excluding=(0, 1, 2)):
     try:
         MAXFD = os.sysconf("SC_OPEN_MAX")
-    except:
+    except ValueError:
         MAXFD = 256
     for i in range(0, MAXFD):
         if i not in excluding:
             try:
                 os.close(i)
-            except:
+            except (IOError, OSError):
                 pass
 
 def get_all_fds():
@@ -414,7 +474,7 @@ def get_all_fds():
     sys.stderr.write("Uh-oh, can't close fds, please port me to your system...\n")
     return fds
 
-def close_all_fds(exceptions=[]):
+def close_all_fds(exceptions=()):
     for fd in get_all_fds():
         try:
             if fd not in exceptions:
@@ -424,12 +484,19 @@ def close_all_fds(exceptions=[]):
             # by listdir() is already closed.
             pass
 
+def use_tty():
+    from xpra.util import envbool
+    NOTTY = envbool("XPRA_NOTTY", False)
+    stdin = sys.stdin
+    return not os.environ.get("MSYSCON") and not NOTTY and stdin and stdin.isatty()
 
-def shellsub(s, subs={}):
+
+def shellsub(s, subs=None):
     """ shell style string substitution using the dictionary given """
-    for var,value in subs.items():
-        s = s.replace("$%s" % var, str(value))
-        s = s.replace("${%s}" % var, str(value))
+    if subs:
+        for var,value in subs.items():
+            s = s.replace("$%s" % var, str(value))
+            s = s.replace("${%s}" % var, str(value))
     return s
 
 
@@ -439,15 +506,11 @@ def osexpand(s, actual_username="", uid=0, gid=0, subs={}):
             #replace "~/" with "~$actual_username/"
             return os.path.expanduser("~%s/%s" % (actual_username, s[2:]))
         return os.path.expanduser(s)
-    if actual_username:
-        HOME = os.path.expanduser("~%s" % actual_username)
-    else:
-        HOME = os.path.expanduser("~")
     from collections import OrderedDict
     d = OrderedDict(subs)
     d.update({
         "PID"   : os.getpid(),
-        "HOME"  : HOME,
+        "HOME"  : expanduser("~/"),
         })
     if os.name=="posix":
         d.update({
@@ -458,10 +521,8 @@ def osexpand(s, actual_username="", uid=0, gid=0, subs={}):
             from xpra.platform.xposix.paths import get_runtime_dir
             d["XDG_RUNTIME_DIR"] = os.environ.get("XDG_RUNTIME_DIR", get_runtime_dir())
     if actual_username:
-        d.update({
-            "USERNAME"  : actual_username,
-            "USER"      : actual_username,
-            })
+        d["USERNAME"] = actual_username
+        d["USER"] = actual_username
     #first, expand the substitutions themselves,
     #as they may contain references to other variables:
     ssub = OrderedDict()
@@ -482,7 +543,8 @@ def path_permission_info(filename, ftype=None):
             if os.path.isdir(filename):
                 ftype = "directory"
         info.append("permissions on %s %s: %s" % (ftype, filename, oct(stat.S_IMODE(stat_info.st_mode))))
-        import pwd,grp      #@UnresolvedImport
+        import pwd
+        import grp      #@UnresolvedImport
         user = pwd.getpwuid(stat_info.st_uid)[0]
         group = grp.getgrgid(stat_info.st_gid)[0]
         info.append("ownership %s:%s" % (user, group))
@@ -545,6 +607,9 @@ class OSEnvContext(object):
 
 class FDChangeCaptureContext(object):
 
+    def __init__(self):
+        self.enter_fds = []
+        self.exit_fds = []
     def __enter__(self):
         self.enter_fds = get_all_fds()
     def __exit__(self, *_args):
@@ -566,6 +631,38 @@ class DummyContextManager(object):
         return "DummyContextManager"
 
 
+#workaround incompatibility between paramiko and gssapi:
+class nomodule_context(object):
+
+    def __init__(self, module_name):
+        self.module_name = module_name
+    def __enter__(self):
+        self.saved_module = sys.modules.get(self.module_name)
+        sys.modules[self.module_name] = None
+    def __exit__(self, *_args):
+        if sys.modules.get(self.module_name) is None:
+            if self.saved_module is None:
+                try:
+                    del sys.modules[self.module_name]
+                except KeyError:
+                    pass
+            else:
+                sys.modules[self.module_name] = self.saved_module
+    def __repr__(self):
+        return "nomodule_context(%s)" % self.module_name
+
+class umask_context(object):
+
+    def __init__(self, umask):
+        self.umask = umask
+    def __enter__(self):
+        self.orig_umask = os.umask(self.umask)
+    def __exit__(self, *_args):
+        os.umask(self.orig_umask)
+    def __repr__(self):
+        return "umask_context(%s)" % self.umask
+
+
 def disable_stdout_buffering():
     import gc
     # Appending to gc.garbage is a way to stop an object from being
@@ -636,7 +733,11 @@ def pollwait(process, timeout=5):
 
 def which(command):
     from distutils.spawn import find_executable
-    return find_executable(command)
+    try:
+        return find_executable(command)
+    except Exception:
+        get_util_logger().debug("find_executable(%s)", command, exc_info=True)
+        return None
 
 def get_status_output(*args, **kwargs):
     import subprocess
@@ -681,7 +782,7 @@ def setuidgid(uid, gid):
             os.initgroups(username, gid)
         else:
             import grp      #@UnresolvedImport
-            groups = [gr.gr_gid for gr in grp.getgrall() if (username in gr.gr_mem)]
+            groups = [gr.gr_gid for gr in grp.getgrall() if username in gr.gr_mem]
             os.setgroups(groups)
     #change uid and gid:
     try:
@@ -712,11 +813,11 @@ def get_peercred(sock):
         log = get_util_logger()
         try:
             import socket
-            creds = sock.getsockopt(socket.SOL_SOCKET, SO_PEERCRED, struct.calcsize('3i'))
-            pid, uid, gid = struct.unpack('3i',creds)
+            creds = sock.getsockopt(socket.SOL_SOCKET, SO_PEERCRED, struct.calcsize(b'3i'))
+            pid, uid, gid = struct.unpack(b'3i',creds)
             log("peer: %s", (pid, uid, gid))
             return pid, uid, gid
-        except Exception as  e:
+        except IOError as  e:
             log("getsockopt", exc_info=True)
             log.error("Error getting peer credentials: %s", e)
             return None
