@@ -1,6 +1,6 @@
 /*
  * This file is part of Xpra.
- * Copyright (C) 2016-2018 Antoine Martin <antoine@devloop.org.uk>
+ * Copyright (C) 2016-2018 Antoine Martin <antoine@xpra.org>
  * Copyright (c) 2016 Spikes, Inc.
  * Licensed under MPL 2.1, see:
  * http://www.mozilla.org/MPL/2.1/
@@ -10,8 +10,11 @@
 'use strict';
 
 var Utilities = {
-	VERSION	: "2.4.3",
+	VERSION	: "2.5.1",
 
+	exc : function() {
+		console.error.apply(console, arguments);
+	},
 	error : function() {
 		console.error.apply(console, arguments);
 	},
@@ -21,6 +24,26 @@ var Utilities = {
 	log : function() {
 		console.log.apply(console, arguments);
 	},
+	debug : function() {
+		console.debug.apply(console, arguments);
+	},
+
+	//these versions should not be redirected:
+	cexc : function() {
+		console.error.apply(console, arguments);
+	},
+	cerror : function() {
+		console.error.apply(console, arguments);
+	},
+	cwarn : function() {
+		console.log.apply(console, arguments);
+	},
+	clog : function() {
+		console.log.apply(console, arguments);
+	},
+	cdebug : function() {
+		console.debug.apply(console, arguments);
+	},
 
 	getHexUUID: function() {
 		var s = [];
@@ -201,6 +224,9 @@ var Utilities = {
 		var ua = navigator.userAgent.toLowerCase();
 		return ua.includes("safari") && !ua.includes('chrome');
 	},
+	isEdge : function() {
+		return navigator.userAgent.includes("Edge");
+	},	
 	isChrome : function () {
 		var isChromium = window.chrome,
 			winNav = window.navigator,
@@ -433,8 +459,13 @@ var Utilities = {
 			};
 		}
 		var value = getParameter(prop);
-		if (value === undefined && typeof(sessionStorage) !== undefined) {
-			value = sessionStorage.getItem(prop);
+		try {
+			if (value === undefined && typeof(sessionStorage) !== undefined) {
+				value = sessionStorage.getItem(prop);
+			}
+		}
+		catch (e) {
+			value = null;
 		}
 		return value;
 	},
@@ -448,6 +479,22 @@ var Utilities = {
 		return ["true", "on", "1", "yes", "enabled"].indexOf(String(v).toLowerCase())!==-1;
 	},
 
+	hasSessionStorage : function() {
+		if (typeof(Storage) === "undefined") {
+			return false;
+		}
+		try {
+			var key = "just for testing sessionStorage support";
+		    sessionStorage.setItem(key, "store-whatever");
+		    sessionStorage.removeItem(key);
+		    return true;
+		}
+		catch (e) {
+			return false;
+		}
+	},
+
+
 	getConnectionInfo : function() {
 		var c = navigator.connection;
 		if (!c) {
