@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2017 Antoine Martin <antoine@devloop.org.uk>
+ * Copyright (c) 2013-2017 Antoine Martin <antoine@xpra.org>
  * Copyright (c) 2014 Joshua Higgins <josh@kxes.net>
  * Copyright (c) 2015-2016 Spikes, Inc.
  * Licensed under MPL 2.0
@@ -28,6 +28,7 @@ function XpraWindow(client, canvas_state, wid, x, y, w, h, metadata, override_re
 	this.log = function() { client.log.apply(client, arguments); };
 	this.warn = function() { client.warn.apply(client, arguments); };
 	this.error = function() { client.error.apply(client, arguments); };
+	this.exc = function() { client.exc.apply(client, arguments); };
 	this.debug = function() { client.debug.apply(client, arguments); };
 	this.debug_categories = client.debug_categories;
 	//keep reference both the internal canvas and screen drawn canvas:
@@ -112,7 +113,7 @@ function XpraWindow(client, canvas_state, wid, x, y, w, h, metadata, override_re
 		jQuery(this.div).addClass("window-" + this.windowtype);
 	}
 
-	if (this.client.server_is_desktop) {
+	if (this.client.server_is_desktop || this.client.server_is_shadow) {
 		jQuery(this.div).addClass("desktop");
 		this.resizable = false;
 	}
@@ -192,11 +193,48 @@ function XpraWindow(client, canvas_state, wid, x, y, w, h, metadata, override_re
 		div.addEventListener('DOMMouseScroll',	on_mousescroll, false); // for Firefox
 	}
 
-	// need to update the CSS geometry
-	this.ensure_visible();
-	this.updateCSSGeometry();
-	//show("placing new window at "+this.x+","+this.y);
+	this.pointer_down = -1;
+	this.pointer_last_x = 0;
+	this.pointer_last_y = 0;
+	if (window.PointerEvent) {
+		this.canvas.addEventListener("pointerdown", function(ev) {
+			me.debug("mouse", "pointerdown:", ev);
+			if (ev.pointerType=="touch") {
+				me.pointer_down = ev.pointerId;
+				me.pointer_last_x = ev.offsetX;
+				me.pointer_last_y = ev.offsetY;
+			}
+		});
+		this.canvas.addEventListener("pointermove", function(ev) {
+			me.debug("mouse", "pointermove:", ev);
+			if (me.pointer_down==ev.pointerId) {
+				var dx = ev.offsetX-me.pointer_last_x;
+				var dy = ev.offsetY-me.pointer_last_y;
+				me.pointer_last_x = ev.offsetX;
+				me.pointer_last_y = ev.offsetY;
+				var mult = 20.0*(window.devicePixelRatio || 1);
+				ev.wheelDeltaX = Math.round(dx*mult);
+				ev.wheelDeltaY = Math.round(dy*mult);
+				on_mousescroll(ev);
+			}
+		});
+		this.canvas.addEventListener("pointerup", function(ev) {
+			me.debug("mouse", "pointerup:", ev);
+			me.pointer_down = -1;
+		});
+		this.canvas.addEventListener("pointercancel", function(ev) {
+			me.debug("mouse", "pointercancel:", ev);
+			me.pointer_down = -1;
+		});
+		this.canvas.addEventListener("pointerout", function(ev) {
+			me.debug("mouse", "pointerout:", ev);
+		});
+	}
 
+	// adapt to screen size if needed (ie: shadow / desktop windows):
+	this.screen_resized();
+	// set the CSS geometry
+	this.updateCSSGeometry();
 	// now read all metadata
 	this.update_metadata(metadata);
 };
@@ -224,6 +262,11 @@ XpraWindow.prototype.set_spinner = function(state) {
 }
 
 XpraWindow.prototype.ensure_visible = function() {
+	if (this.client.server_is_desktop || this.client.server_is_shadow) {
+		//those windows should usually be centered on screen,
+		//moving them would mess that up
+		return;
+	}
 	var oldx = this.x;
 	var oldy = this.y;
 	// for now make sure we don't out of top left
@@ -245,6 +288,7 @@ XpraWindow.prototype.ensure_visible = function() {
 	else if (oldy >= wh - min_visible) {
 		this.y = Math.min(oldy, wh - min_visible);
 	}
+	this.debug("geometry", "ensure_visible() oldx=", oldx, "oldy=", oldy, "x=", this.x, "y=", this.y);
 	if((oldx != this.x) || (oldy != this.y)) {
 		this.updateCSSGeometry();
 		return false;
@@ -271,7 +315,7 @@ XpraWindow.prototype.updateCanvasGeometry = function() {
 XpraWindow.prototype.updateCSSGeometry = function() {
 	// set size of canvas
 	this.updateCanvasGeometry();
-	if (this.client.server_is_desktop) {
+	if (this.client.server_is_desktop || this.client.server_is_shadow) {
 		jQuery(this.div).position({of : jQuery("#screen")});
 		return;
 	}
@@ -286,6 +330,7 @@ XpraWindow.prototype.updateCSSGeometry = function() {
 	this.outerY = this.y - this.topoffset;
 	jQuery(this.div).css('left', this.outerX);
 	jQuery(this.div).css('top', this.outerY);
+	this.debug("geometry", "updateCSSGeometry() left=", this.outerX, ", top=", this.outerY, ", width=", this.outerW, ", height=", this.outerH);
 }
 
 XpraWindow.prototype.updateFocus = function() {
@@ -322,7 +367,7 @@ XpraWindow.prototype.on_mouseup = function(e) {
 
 XpraWindow.prototype.on_mousescroll = function(e) {
 	this.mouse_scroll_cb(this.client, e, this);
-	e.preventDefault();
+	//e.preventDefault();
 	return false;
 }
 
@@ -339,7 +384,7 @@ XpraWindow.prototype.update_zindex = function() {
 	if (this.tray) {
 		z = 0;
 	}
-	else if (this.override_redirect || this.client.server_is_desktop) {
+	else if (this.override_redirect || this.client.server_is_desktop || this.client.server_is_shadow) {
 		z = 15000;
 	}
 	else if (this.windowtype=="DROPDOWN" || this.windowtype=="TOOLTIP" ||
@@ -412,7 +457,7 @@ XpraWindow.prototype.set_metadata_safe = function(metadata) {
 			opacity = 1.0;
 		}
 		else {
-			opacity = opacity / 0x100000000
+			opacity = opacity / 0x100000000;
 		}
 		jQuery(this.div).css('opacity', ''+opacity);
 	}
@@ -440,7 +485,7 @@ XpraWindow.prototype.set_metadata_safe = function(metadata) {
 			//remove any existing "wmclass-" classes not in the new wm_class list:
 			for (var i = 0; i < classes.length; i++) {
 				var tclass = ""+classes[i];
-				if (tclass.indexOf("wmclass-")===0 && wm_class && wm_class.indexOf(tclass)<0) {
+				if (tclass.indexOf("wmclass-")===0 && wm_class && !wm_class.includes(tclass)) {
 					jQuery(this.div).removeClass(tclass);
 				}
 			}
@@ -458,7 +503,6 @@ XpraWindow.prototype.set_metadata_safe = function(metadata) {
 };
 
 XpraWindow.prototype.apply_size_constraints = function() {
-	var size_constraints = this.metadata["size-constraints"];
 	if (!this.resizable) {
 		return;
 	}
@@ -474,6 +518,7 @@ XpraWindow.prototype.apply_size_constraints = function() {
 		hdec = jQuery('#head' + this.wid).outerHeight(true);
 	}
 	var min_size = null, max_size = null;
+	var size_constraints = this.metadata["size-constraints"];
 	if (size_constraints) {
 		min_size = size_constraints["minimum-size"];
 		max_size = size_constraints["maximum-size"];
@@ -533,6 +578,7 @@ XpraWindow.prototype.save_geometry = function() {
 			"y"	: this.y,
 			"w"	: this.w,
 			"h" : this.h};
+	this.debug("geometry", "save_geometry() saved-geometry=", this.saved_geometry);
 }
 /**
  * Restores the saved geometry (if it exists).
@@ -545,6 +591,7 @@ XpraWindow.prototype.restore_geometry = function() {
 	this.y = this.saved_geometry["y"];
 	this.w = this.saved_geometry["w"];
 	this.h = this.saved_geometry["h"];
+	this.debug("geometry", "restore_geometry() saved-geometry=", this.saved_geometry);
 	// delete saved geometry
 	this.saved_geometry = null;
 	// then call local resized callback
@@ -578,6 +625,11 @@ XpraWindow.prototype.toggle_maximized = function() {
  * Fullscreen / unfullscreen the window.
  */
 XpraWindow.prototype.set_fullscreen = function(fullscreen) {
+	//the browser itself:
+	//we can't bring attention to the fullscreen widget, ie:
+	//$("#fullscreen").fadeIn(100).fadeOut(100).fadeIn(100).fadeOut(100).fadeIn(100);
+	//because the window is about to cover the top bar...
+	//so just fullscreen the window:
 	if (this.fullscreen==fullscreen) {
 		return;
 	}
@@ -605,6 +657,7 @@ XpraWindow.prototype._set_decorated = function(decorated) {
 		jQuery(this.div).addClass("window");
 		if (this.d_header) {
 			this.topoffset = this.topoffset + parseInt(jQuery(this.d_header).css('height'), 10);
+			this.debug("geometry", "_set_decorated(", decorated, ") new topoffset=", self.topoffset);
 		}
 	}
 	else {
@@ -641,6 +694,7 @@ XpraWindow.prototype.fill_screen = function() {
 	this.y = 0 + this.topoffset;
 	this.w = (screen_size[0] - this.leftoffset) - this.rightoffset;
 	this.h = (screen_size[1] - this.topoffset) - this.bottomoffset;
+	this.debug("geometry", "fill_screen() ", this.x, this.y, this.w, this.h);
 };
 
 
@@ -655,6 +709,7 @@ XpraWindow.prototype.handle_resized = function(e) {
 	// this function is called on local resize only,
 	// remote resize will call this.resize()
 	// need to update the internal geometry
+	this.debug("geometry", "handle_resized(", e, ")");
 	if(e) {
 		this.x = this.x + Math.round(e.position.left - e.originalPosition.left);
 		this.y = this.y + Math.round(e.position.top - e.originalPosition.top);
@@ -672,11 +727,14 @@ XpraWindow.prototype.handle_resized = function(e) {
  * store internal geometry, external is always in CSS left and top
  */
 XpraWindow.prototype.handle_moved = function(e) {
+	var left = Math.round(e.position.left),
+		top = Math.round(e.position.top);
+	this.debug("geometry", "handle_moved(", e, ") left=", left, ", top=", top);
 	// add on padding to the event position so that
 	// it reflects the internal geometry of the canvas
 	//this.log("handle moved: position=", e.position.left, e.position.top);
-	this.x = Math.round(e.position.left) + this.leftoffset;
-	this.y = Math.round(e.position.top) + this.topoffset;
+	this.x = left + this.leftoffset;
+	this.y = top + this.topoffset;
 	// make sure we are visible after move
 	this.ensure_visible();
 	// tell remote we have moved window
@@ -688,10 +746,18 @@ XpraWindow.prototype.handle_moved = function(e) {
  * if it is fullscreen or maximized.
  */
 XpraWindow.prototype.screen_resized = function() {
-	console.log("screen resized");
+	this.debug("geometry", "screen_resized() server_is_desktop=", this.client.server_is_desktop, ", server_is_shadow=", this.client.server_is_shadow);
 	if (this.client.server_is_desktop) {
 		this.match_screen_size();
-		this.handle_resized();
+	}
+	if (this.client.server_is_shadow) {
+		//note: when this window is created,
+		// it may not have been added to the client's list yet
+		var ids = Object.keys(this.client.id_to_window);
+		if (ids.length==0 || ids[0]==this.wid) {
+			//single window, recenter it:
+			this.recenter();
+		}
 	}
 	if (this.fullscreen || this.maximized) {
 		this.fill_screen();
@@ -700,6 +766,28 @@ XpraWindow.prototype.screen_resized = function() {
 	this.ensure_visible();
 };
 
+XpraWindow.prototype.recenter = function() {
+	var x = this.x,
+		y = this.y;
+	this.debug("geometry", "recenter() x=", x, ", y=", y, ", desktop size: ", this.client.desktop_width, this.client.desktop_height);
+	x = Math.round((this.client.desktop_width-this.w)/2);
+	y = Math.round((this.client.desktop_height-this.h)/2);
+	if (this.x!=x || this.y!=y) {
+		this.debug("geometry", "window re-centered to:", x, y);
+		this.x = x;
+		this.y = y;
+		this.updateCSSGeometry();
+		this.geometry_cb(this);
+	}
+	else {
+		this.debug("geometry", "recenter() unchanged at ", x, y);
+	}
+	if (this.x<0 || this.y<0) {
+		this.warn("window does not fit in canvas, offsets: ", x, y);
+	}
+}
+
+
 XpraWindow.prototype.match_screen_size = function() {
 	var maxw = this.client.desktop_width;
 	var maxh = this.client.desktop_height;
@@ -707,10 +795,11 @@ XpraWindow.prototype.match_screen_size = function() {
 	if (this.client.server_resize_exact) {
 		neww = maxw;
 		newh = maxh;
-		console.log("resizing to exact size:", neww, newh);
+		this.log("resizing to exact size:", neww, newh);
 	}
 	else {
 		if (this.client.server_screen_sizes.length==0) {
+			this.recenter();
 			return;
 		}
 		//try to find the best screen size to use,
@@ -730,7 +819,7 @@ XpraWindow.prototype.match_screen_size = function() {
 			}
 		}
 		if (neww==0 && newh==0) {
-			//not found, try to fine the smallest one:
+			//not found, try to find the smallest one:
 			best = 0;
 			for (var i = 0; i < screen_sizes.length; i++) {
 				screen_size = screen_sizes[i];
@@ -743,9 +832,9 @@ XpraWindow.prototype.match_screen_size = function() {
 				}
 			}
 		}
-		console.log("best screen size:", neww, newh);
+		this.log("best screen size:", neww, newh);
 	}
-	this.resize(neww, newh);
+	this.recenter();
 };
 
 
@@ -754,6 +843,7 @@ XpraWindow.prototype.match_screen_size = function() {
  */
 
 XpraWindow.prototype.move_resize = function(x, y, w, h) {
+	this.debug("geometry", "move_resize(", x, y, w, h, ")");
 	// only do it if actually changed!
 	if(!(this.w == w) || !(this.h == h) || !(this.x == x) || !(this.y == y)) {
 		this.w = w;
@@ -770,10 +860,12 @@ XpraWindow.prototype.move_resize = function(x, y, w, h) {
 };
 
 XpraWindow.prototype.move = function(x, y) {
+	this.debug("geometry", "move(", x, y, ")");
 	this.move_resize(x, y, this.w, this.h);
 };
 
 XpraWindow.prototype.resize = function(w, h) {
+	this.debug("geometry", "resize(", w, h, ")");
 	this.move_resize(this.x, this.y, w, h);
 };
 
@@ -857,6 +949,12 @@ XpraWindow.prototype.set_cursor = function(encoding, w, h, xhot, yhot, img_data)
 };
 
 
+XpraWindow.prototype.eos = function() {
+	this._close_jsmpeg();
+	this._close_broadway();
+	this._close_video();
+}
+
 
 /**
  * This function draws the contents of the off-screen canvas to the visible
@@ -885,22 +983,27 @@ XpraWindow.prototype._init_broadway = function(enc_width, enc_height, width, hei
 	this.log("broadway decoder initialized");
 	this.broadway_paint_location = [0, 0];
 	this.broadway_decoder.onPictureDecoded = function(buffer, p_width, p_height, infos) {
-		me.debug("draw", "broadway picture decoded: ", buffer.length, "bytes, size ", p_width, "x", p_height+", paint location: ", me.broadway_paint_location,"with infos=", infos);
-		if(!me.broadway_decoder) {
-			return;
-		}
-		var img = me.offscreen_canvas_ctx.createImageData(p_width, p_height);
-		img.data.set(buffer);
-		var x = me.broadway_paint_location[0];
-		var y = me.broadway_paint_location[1];
-		me.offscreen_canvas_ctx.putImageData(img, x, y);
-		if(enc_width!=width || enc_height!=height) {
-			//scale it:
-			me.offscreen_canvas_ctx.drawImage(me.offscreen_canvas, x, y, p_width, p_height, x, y, width, height);
-		}
+		me._broadway_paint(buffer, enc_width, enc_height, width, height, p_width, p_height, infos);
 	};
 };
 
+XpraWindow.prototype._broadway_paint = function(buffer, enc_width, enc_height, width, height, p_width, p_height, infos) {
+	this.debug("draw", "broadway picture decoded: ", buffer.length, "bytes, size ", p_width, "x", p_height+", paint location: ", this.broadway_paint_location,"with infos=", infos);
+	if(!this.broadway_decoder) {
+		return;
+	}
+	var img = this.offscreen_canvas_ctx.createImageData(p_width, p_height);
+	img.data.set(buffer);
+	var x = this.broadway_paint_location[0];
+	var y = this.broadway_paint_location[1];
+	this.offscreen_canvas_ctx.putImageData(img, x, y, 0, 0, enc_width, enc_height);
+	if(enc_width!=width || enc_height!=height) {
+		//scale it:
+		this.offscreen_canvas_ctx.drawImage(this.offscreen_canvas, x, y, enc_width, enc_height, x, y, width, height);
+	}
+};
+
+
 XpraWindow.prototype._close_broadway = function() {
 	this.broadway_decoder = null;
 }
@@ -917,7 +1020,7 @@ XpraWindow.prototype._close_video = function() {
 				}
 				this.media_source.endOfStream();
 			} catch(e) {
-				this.warn("video media source EOS error: "+e);
+				this.exc(e, "video media source EOS error");
 			}
 			this.video_source_buffer = null;
 			this.media_source = null;
@@ -992,7 +1095,7 @@ XpraWindow.prototype._init_video = function(width, height, coding, profile, leve
 		codec_string = 'video/webm;codecs="vp9"';
 	}
 	else {
-		throw Exception("invalid encoding: "+coding);
+		throw new Error("invalid encoding: "+coding);
 	}
 	this.log("video codec string: "+codec_string+" for "+coding+" profile '"+profile+"', level '"+level+"'");
 	this.media_source.addEventListener('sourceopen', function() {
@@ -1068,8 +1171,31 @@ var DEFAULT_BOX_COLORS = {
         "vp9"     : "lavender",
         "mpeg4"   : "black",
         "scroll"  : "brown",
+        "mpeg1"   : "olive",
         }
 
+XpraWindow.prototype.get_jsmpeg_renderer = function get_jsmpeg_renderer() {
+	if (this.jsmpeg_renderer==null) {
+		var options = new Object();
+		//webgl is still buggy
+		//if (JSMpeg.Renderer.WebGL.IsSupported()) {
+		//	this.jsmpeg_renderer = new JSMpeg.Renderer.WebGL(options);
+		//}
+		//else {
+		this.jsmpeg_renderer = new JSMpeg.Renderer.Canvas2D(options);
+		//}
+	}
+	return this.jsmpeg_renderer;
+}
+
+XpraWindow.prototype._close_jsmpeg = function _close_jsmpeg() {
+	if (this.jsmpeg_renderer!=null) {
+		this.jsmpeg_renderer.destroy();
+	}
+	//decoder doesn't need cleanup?
+	this.jsmpeg_decoder = null;
+}
+
 XpraWindow.prototype.do_paint = function paint(x, y, width, height, coding, img_data, packet_sequence, rowstride, options, decode_callback) {
 	this.debug("draw", "do_paint(", img_data.length, " bytes of ", ("zlib" in options?"zlib ":""), coding, " data ", width, "x", height, " at ", x, ",", y, ") focused=", this.focused);
 	var me = this;
@@ -1088,6 +1214,7 @@ XpraWindow.prototype.do_paint = function paint(x, y, width, height, coding, img_
 	}
 
 	function painted(skip_box) {
+		img_data = null;
 		me.paint_pending = 0;
 		if (me.paint_debug && !skip_box) {
 			var color = DEFAULT_BOX_COLORS[coding] || "white";
@@ -1097,6 +1224,7 @@ XpraWindow.prototype.do_paint = function paint(x, y, width, height, coding, img_
 	}
 
 	function paint_error(e) {
+		img_data = null;
 		me.error("error painting", coding, e);
 		me.paint_pending = 0;
 		decode_callback(""+e);
@@ -1161,6 +1289,36 @@ XpraWindow.prototype.do_paint = function paint(x, y, width, height, coding, img_
 			}
 			j.src = "data:image/"+coding+";base64," + Utilities.ArrayBufferToBase64(img_data);
 		}
+		else if (coding=="mpeg1") {
+			var frame = options["frame"] || 0;
+			if (frame==0 || this.jsmpeg_decoder==null) {
+				var options = new Object();
+				options.streaming = true;
+				options.decodeFirstFrame = false;
+				this.jsmpeg_decoder = new JSMpeg.Decoder.MPEG1Video(options);
+				//TODO: instead of delegating, we should probably subclass the renderer
+				// (but which one! GL or not?):
+				var renderer = new Object();
+				renderer.render = function render(Y, Cr, Cb) {
+					var jsmpeg_renderer = me.get_jsmpeg_renderer();
+					jsmpeg_renderer.render(Y, Cr, Cb);
+					var canvas = jsmpeg_renderer.canvas;
+					me.offscreen_canvas_ctx.drawImage(canvas, x, y, width, height);
+					paint_box("olive", x, y, width, height);
+				}
+				renderer.resize = function resize(newWidth, newHeight) {
+					var jsmpeg_renderer = me.get_jsmpeg_renderer();
+					jsmpeg_renderer.resize(newWidth, newHeight);
+				}
+				this.jsmpeg_decoder.connect(renderer);
+			}
+			var pts = frame;
+			this.jsmpeg_decoder.write(pts, img_data);
+			var decoded = this.jsmpeg_decoder.decode();
+			this.debug("draw", coding, "frame", frame, "data len=", img_data.length, "decoded=", decoded);
+			//TODO: only call painted when we have actually painted the frame?
+			painted();
+		}
 		else if (coding=="h264") {
 			var frame = options["frame"] || 0;
 			if(frame==0) {
@@ -1235,6 +1393,7 @@ XpraWindow.prototype.do_paint = function paint(x, y, width, height, coding, img_
 		}
 	}
 	catch (e) {
+		this.exc(e, "error painting", coding, "sequence no", packet_sequence);
 		paint_error(e);
 	}
 };
@@ -1244,6 +1403,7 @@ XpraWindow.prototype.do_paint = function paint(x, y, width, height, coding, img_
  */
 XpraWindow.prototype.destroy = function destroy() {
 	// remove div
+	this._close_jsmpeg();
 	this._close_broadway();
 	this._close_video();
 	this.div.remove();
